Package: haven
Title: Import and Export 'SPSS', 'Stata' and 'SAS' Files
Version: 2.5.4
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre")),
    person("Evan", "Miller", role = c("aut", "cph"),
           comment = "Author of included ReadStat code"),
    person("Danny", "Smith", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Import foreign statistical formats into R via the embedded
    'ReadStat' C library, <https://github.com/WizardMac/ReadStat>.
License: MIT + file LICENSE
URL: https://haven.tidyverse.org, https://github.com/tidyverse/haven,
        https://github.com/WizardMac/ReadStat
BugReports: https://github.com/tidyverse/haven/issues
Depends: R (>= 3.6)
Imports: cli (>= 3.0.0), forcats (>= 0.2.0), hms, lifecycle, methods,
        readr (>= 0.1.0), rlang (>= 0.4.0), tibble, tidyselect, vctrs
        (>= 0.3.0)
Suggests: covr, crayon, fs, knitr, pillar (>= 1.4.0), rmarkdown,
        testthat (>= 3.0.0), utf8
LinkingTo: cpp11
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.2.3
SystemRequirements: GNU make, zlib: zlib1g-dev (deb), zlib-devel (rpm)
NeedsCompilation: yes
Packaged: 2023-11-30 13:28:43 UTC; hadleywickham
Author: Hadley Wickham [aut, cre],
  Evan Miller [aut, cph] (Author of included ReadStat code),
  Danny Smith [aut],
  Posit Software, PBC [cph, fnd]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2023-11-30 15:10:02 UTC
Built: R 4.5.2; x86_64--netbsd; 2026-02-18 02:29:06 UTC; unix
