Package: haven
Title: Import and Export 'SPSS', 'Stata' and 'SAS' Files
Version: 1.1.1
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre")),
    person("Evan", "Miller", 
      role = c("aut", "cph"),
      comment = "Author of included ReadStat code"
    ),
    person("RStudio", role = c("cph", "fnd"))
  )
Description: Import foreign statistical formats into R via the embedded
    'ReadStat' C library, <https://github.com/WizardMac/ReadStat>.
License: MIT + file LICENSE
URL: http://haven.tidyverse.org, https://github.com/tidyverse/haven,
        https://github.com/WizardMac/ReadStat
BugReports: https://github.com/tidyverse/haven/issues
Depends: R (>= 3.1)
Imports: forcats (>= 0.2.0), hms, Rcpp (>= 0.11.4), readr (>= 0.1.0),
        tibble
Suggests: covr, knitr, rmarkdown, testthat
LinkingTo: Rcpp
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.0.1
SystemRequirements: GNU make
NeedsCompilation: yes
Packaged: 2018-01-17 20:37:09 UTC; hadley
Author: Hadley Wickham [aut, cre],
  Evan Miller [aut, cph] (Author of included ReadStat code),
  RStudio [cph, fnd]
Maintainer: Hadley Wickham <hadley@rstudio.com>
Repository: CRAN
Date/Publication: 2018-01-18 10:31:31 UTC
Built: R 3.4.1; x86_64-w64-mingw32; 2018-03-10 17:03:51 UTC; windows
Archs: x64
