# Generated by roxygen2: do not edit by hand

S3method("[",labelled)
S3method(as.data.frame,labelled)
S3method(as_factor,character)
S3method(as_factor,data.frame)
S3method(as_factor,factor)
S3method(as_factor,labelled)
S3method(is.na,labelled_spss)
S3method(print,labelled)
S3method(print,labelled_spss)
S3method(type_sum,labelled)
S3method(zap_formats,data.frame)
S3method(zap_formats,default)
S3method(zap_labels,data.frame)
S3method(zap_labels,default)
S3method(zap_labels,labelled)
S3method(zap_labels,labelled_spss)
S3method(zap_missing,data.frame)
S3method(zap_missing,default)
S3method(zap_missing,labelled)
S3method(zap_missing,labelled_spss)
export(as_factor)
export(format_tagged_na)
export(is.labelled)
export(is_tagged_na)
export(labelled)
export(labelled_spss)
export(na_tag)
export(print_labels)
export(print_tagged_na)
export(read_dta)
export(read_por)
export(read_sas)
export(read_sav)
export(read_spss)
export(read_stata)
export(tagged_na)
export(write_dta)
export(write_sas)
export(write_sav)
export(zap_empty)
export(zap_formats)
export(zap_labels)
export(zap_missing)
importFrom(Rcpp,sourceCpp)
importFrom(tibble,tibble)
importFrom(tibble,type_sum)
useDynLib(haven)
useDynLib(haven,is_tagged_na_)
useDynLib(haven,na_tag_)
useDynLib(haven,tagged_na_)
