# Generated by roxygen2: do not edit by hand

S3method("[",haven_labelled)
S3method("[",haven_labelled_spss)
S3method(as.data.frame,haven_labelled)
S3method(as_factor,data.frame)
S3method(as_factor,haven_labelled)
S3method(as_factor,labelled)
S3method(format,pillar_shaft_haven_labelled_chr)
S3method(format,pillar_shaft_haven_labelled_num)
S3method(is.na,haven_labelled_spss)
S3method(print,haven_labelled)
S3method(print,haven_labelled_spss)
S3method(type_sum,haven_labelled)
S3method(zap_formats,data.frame)
S3method(zap_formats,default)
S3method(zap_label,data.frame)
S3method(zap_label,default)
S3method(zap_label,haven_labelled)
S3method(zap_labels,data.frame)
S3method(zap_labels,default)
S3method(zap_labels,haven_labelled)
S3method(zap_labels,haven_labelled_spss)
S3method(zap_missing,data.frame)
S3method(zap_missing,default)
S3method(zap_missing,haven_labelled)
S3method(zap_missing,haven_labelled_spss)
S3method(zap_widths,data.frame)
S3method(zap_widths,default)
export(as_factor)
export(format_tagged_na)
export(is.labelled)
export(is_tagged_na)
export(labelled)
export(labelled_spss)
export(na_tag)
export(print_labels)
export(print_tagged_na)
export(read_dta)
export(read_por)
export(read_sas)
export(read_sav)
export(read_spss)
export(read_stata)
export(read_xpt)
export(tagged_na)
export(write_dta)
export(write_sas)
export(write_sav)
export(write_xpt)
export(zap_empty)
export(zap_formats)
export(zap_label)
export(zap_labels)
export(zap_missing)
export(zap_widths)
importFrom(Rcpp,sourceCpp)
importFrom(forcats,as_factor)
importFrom(hms,hms)
importFrom(tibble,tibble)
importFrom(tibble,type_sum)
useDynLib(haven, .registration = TRUE)
